#include <cppunit/tools/StringTools.h>
#include <cppunit/tools/XmlElement.h>
#include <stdexcept>


CPPUNIT_NS_BEGIN

  
XmlElement::XmlElement( M_STD_STRING elementName,
                        M_STD_STRING content ) 
  : m_name( elementName )
  , m_content( content )
{
}

    
XmlElement::XmlElement( M_STD_STRING elementName,
                        int numericContent )
  : m_name( elementName )
{
  setContent( numericContent );
}


XmlElement::~XmlElement()
{
  Elements::iterator itNode = m_elements.begin();
  while ( itNode != m_elements.end() )
  {
    XmlElement *element = *itNode++;
    delete element;
  }
}


M_STD_STRING 
XmlElement::name() const
{
  return m_name;
}


M_STD_STRING 
XmlElement::content() const
{
  return m_content;
}


void 
XmlElement::setName( const M_STD_STRING &name )
{
  m_name = name;
}


void 
XmlElement::setContent( const M_STD_STRING &content )
{
  m_content = content;
}


void 
XmlElement::setContent( int numericContent )
{
  m_content = StringTools::toString( numericContent );
}


void 
XmlElement::addAttribute( M_STD_STRING attributeName,
                          M_STD_STRING value  )
{
  m_attributes.push_back( Attribute( attributeName, value ) );
}


void 
XmlElement::addAttribute( M_STD_STRING attributeName,
                          int numericValue )
{
  addAttribute( attributeName, StringTools::toString( numericValue ) );
}


void 
XmlElement::addElement( XmlElement *node )
{
  m_elements.push_back( node );
}


int 
XmlElement::elementCount() const
{
  return m_elements.size();
}


XmlElement *
XmlElement::elementAt( int index ) const
{
  if ( index < 0  ||  index >= elementCount() )
    throw std::invalid_argument( "XmlElement::elementAt(), out of range index" );

  return m_elements[ index ];
}


XmlElement *
XmlElement::elementFor( const M_STD_STRING &name ) const
{
  Elements::const_iterator itElement = m_elements.begin();
  for ( ; itElement != m_elements.end(); ++itElement )
  {
    if ( (*itElement)->name() == name )
      return *itElement;
  }

  throw std::invalid_argument( "XmlElement::elementFor(), not matching child element found" );
  return NULL;  // make some compilers happy.
}


M_STD_STRING 
XmlElement::toString( const M_STD_STRING &indent ) const
{
  M_STD_STRING element( indent );
  element += _T("<");
  element += m_name;
  if ( !m_attributes.empty() )
  {
    element += _T(" ");
    element += attributesAsString();
  }
  element += _T(">");

  if ( !m_elements.empty() )
  {
    element += _T("\n");

    M_STD_STRING subNodeIndent( indent + _T("  ") );
    Elements::const_iterator itNode = m_elements.begin();
    while ( itNode != m_elements.end() )
    {
      const XmlElement *node = *itNode++;
      element += node->toString( subNodeIndent );
    }

    element += indent;
  }

  if ( !m_content.empty() )
  {
    element += escape( m_content );
    if ( !m_elements.empty() )
    {
      element += _T("\n");
      element += indent;
    }
  }

  element += _T("</");
  element += m_name;
  element += _T(">\n");

  return element;
}


M_STD_STRING 
XmlElement::attributesAsString() const
{
  M_STD_STRING attributes;
  Attributes::const_iterator itAttribute = m_attributes.begin();
  while ( itAttribute != m_attributes.end() )
  {
    if ( !attributes.empty() )
       attributes += _T(" ");

    const Attribute &attribute = *itAttribute++;
    attributes += attribute.first;
    attributes += _T("=\"");
    attributes += escape( attribute.second );
    attributes += _T("\"");
  }
  return attributes;
}


M_STD_STRING 
XmlElement::escape( M_STD_STRING value ) const
{
  M_STD_STRING escaped;
  for ( unsigned int index =0; index < value.length(); ++index )
  {
    TCHAR c = value[index ];
    switch ( c )    // escape all predefined XML entity (safe?)
    {
    case '<': 
      escaped += _T("&lt;");
      break;
    case '>': 
      escaped += _T("&gt;");
      break;
    case '&': 
      escaped += _T("&amp;");
      break;
    case '\'': 
      escaped += _T("&apos;");
      break;
    case '"': 
      escaped += _T("&quot;");
      break;
    default:
      escaped += c;
    }
  }
  
  return escaped;
}


CPPUNIT_NS_END

